/*
*******************************************************************************
** Copyright (C) 1995 Advanced Processing Laboratories Inc. (AP Labs)
** All rights reserved.
**
** This file and its contents are a product of AP Labs.
** All software distributed by AP Labs is done so under a software license.
** This file may not be copied or modified without execution of the
** appropriate software license agreement with AP Labs or explicit written
** permission of AP Labs.
**
** This file is provided as is, with no warranties of any kind including
** the warranties of design, merchantability and fitness for a particular
** purpose, or arising from a course of dealing, usage or trade practice.
**
** AP Labs shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part thereof.
**
** In no event will AP Labs be liable for any lost revenue or profits
** or other special, indirect and consequential damages, even if AP Labs
** has been advised of the possibility of such damages, as a result of the
** usage of this file and software for which this file is a part.
******************************************************************************
*/

/*
------------------------------------------------------------------------------

File Name:      btfpTest.c

Description:
        Runs the Bancomm TFP driver test.

Modification History:
        Who     Date            What
        ---     ----            ----
        bpw     08/28/95        created
-------------------------------------------------------------------------------
*/

/*
 * Include files
 */
#include "vxWorks.h"
#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "logLib.h"
#include "sysLib.h"
#include "taskLib.h"
#include "bctfp.app.h"

/*
 * Defines
 */
#define PRNT(x)   logMsg((char *)x,0,0,0,0,0,0);
#define HVAL(x,y) logMsg("Value of %s is 0x%x.\n",(int)x,(unsigned int)y,0,0,0,0)
#define DVAL(x,y) logMsg("Value of %s is %d.\n",(int)x,(unsigned int)y,0,0,0,0)
#define CVAL(x,y) logMsg("Value of %s is %c.\n",(int)x,(unsigned char)y,0,0,0,0)
#define SVAL(x,y) logMsg("%s is %s.\n",(int)x,(int)y,0,0,0,0)
#define ERRP(x,y) logMsg("*** Error: %s at %s ***\n",(int)x,(int)y,0,0,0,0)

/*
 * typedefs
 */

/*
 * Imports
 */

/*
 * Exports
 */

/*
 * Locals
 */
int     flag = 0,
        max = 0;
int     fd = 0;
char    buf[1024];

/*
 * Forward declarations
 */
STATUS  go();
VOID    myhandler(int);


VOID
myhandler(int val)
{
    char    tmp[30];
    int     status;
    args    ioctlArgs;

    PRNT("*** called myhandler ***\n");
    if (val & INT_DPA)
    {
        max += read(fd, (char *) &buf[max], 512 - max);
        flag = 1;
    }
    if (val & INT_1PPS)
    {
        ioctl(fd, PACKET, OFF);
        sprintf(tmp, "%c%s%c", SOH, "B123455601", ETB);
        status = write(fd, tmp, 12);
        if (status < 12)
            ERRP("noted", "interrupt write");
        ioctl(fd, PACKET, ON);
        ioctlArgs.reg_id = R_MASK;
        ioctlArgs.reg_val = INT_DPA;
        ioctl(fd, WRITEREG, (int) &ioctlArgs);
    }
}

STATUS
go(int input)
{
    STATUS  status;
    int     val,
            errCount = 0,
            i,
            count = 0;
    args    ioctlArgs;
    static  onceThrough;
    char    tmp[100];
    unsigned short tmp2[30];

    PRNT("----- Bancomm Time Frequency Processor Driver Test -----\n");
    PRNT("Starting test...\n");
    PRNT("Step 1: Calling btfpDevCreate...\n");
    if (onceThrough != 1)
    {
        if (input == 0)
            status = btfpDevCreate("btfp0", (char *) 0xff00, 5, 25);
        else
            status = btfpDevCreate("btfp1", (char *) 0xff80, 7, 249);
        if (status == ERROR)
        {
            ERRP("Couldn't create device", "btfpDevCreate");
            return ERROR;
        }
    }
    onceThrough = 1;

    PRNT("Step 2: Opening Time Frequency Processor...\n");
    if (input == 0)
        fd = open("btfp0", 0, 0);
    else
        fd = open("btfp1", 0, 0);
    if (fd < 0)
    {
        ERRP("Can't open Time Frequency Processor", "open");
        return ERROR;
    }

    PRNT("Step 3: Testing ioctls...\n");

    status = ioctl(fd, DEBUG, 0x0);
    if (status != OK)
    {
        ERRP("found", "DEBUG ioctl");
        return ERROR;
    }

    status = ioctl(fd, PACKET, ON);
    if (status != OK)
    {
        ERRP("found", "PACKET ioctl");
        return ERROR;
    }
    ioctl(fd, PACKET, OFF);

    status = ioctl(fd, SIGNAL, OFF);
    if (status != OK)
    {
        ERRP("found", "SIGNAL ioctl");
        return ERROR;
    }

    status = ioctl(fd, INTCONN, (int) myhandler);
    if (status != OK)
    {
        ERRP("found", "INTCONN ioctl");
        return ERROR;
    }

    status = ioctl(fd, ROUTFIFO, (int) tmp);
    if (status != OK)
    {
        ERRP("found", "ROUTFIFO ioctl");
        return ERROR;
    }

    *(int *) tmp2 = R_ALL;
    status = ioctl(fd, READREG, (int) tmp2);
    if (status != OK)
    {
        ERRP("found", "READREG ALL ioctl");
        return ERROR;
    }

    for (i = 0; i < 21; i++)
        printf("Register %d is 0x%x\n", i, (unsigned int) tmp2[i]);

    status = ioctl(fd, WINFIFO, SOH);
    status = ioctl(fd, WINFIFO, 'A');
    status = ioctl(fd, WINFIFO, '7');
    status = ioctl(fd, WINFIFO, ETB);
    ioctlArgs.reg_id = R_ACK;
    ioctlArgs.reg_val = ACK_INACT;
    status = ioctl(fd, WRITEREG, (int) &ioctlArgs);

    PRNT("Reading time...\n");
    val = read(fd, buf, 17);
    SVAL("buf", buf);
    if (val < 0)
    {
        ERRP("noted", "read");
        errCount++;
    }
    DVAL("bytes read", val);

    taskDelay(sysClkRateGet());

    ioctl(fd, PACKET, ON);

    ioctl(fd, DEBUG, 0x0);

    /* connect a user interrupt handler */
    ioctl(fd, INTCONN, (int) myhandler);

    /* tell driver we want to be notified of interrupts */
    ioctl(fd, SIGNAL, ON);

    /* turn on DPA and 1PPS interrupts */
    ioctlArgs.reg_id = R_MASK;
    ioctlArgs.reg_val = INT_DPA | INT_1PPS;
    ioctl(fd, WRITEREG, (int) &ioctlArgs);

    PRNT("Setting mode to free running...\n");
    sprintf(buf, "%c%s%c", SOH, "A1", ETB);
    val = write(fd, buf, 4);
    if (val < 4)
    {
        ERRP("noted", "write 1");
        errCount++;
    }

    PRNT("Waiting 10 seconds for LED view...\n");
    taskDelay(sysClkRateGet() * 10);

    /* second board can't do GPS.  It is a bc635 */
    if (input == 0)
    {
        PRNT("Going to GPS mode...\n");
        sprintf(buf, "%c%s%c", SOH, "A6", ETB);
        val = write(fd, buf, 4);
        DVAL("val", val);
        if (val < 4)
        {
            ERRP("noted", "write 2");
            errCount++;
        }

        /* change GPS parameters so we can check output */
        /* ioctl(fd,DEBUG,0x7); */
        PRNT("Changing GPS parameters...reducing packet traffic...\n");
        sprintf(buf, "%c%c%c%c%c%c%c%c%c%c%c", SOH, 'J', ETB, DLE, 0x35, 0x02, 0x0, 0x04, 0x0, DLE, ETX);
        val = write(fd, buf, 11);
        max = 0;
        flag = 0;
        DVAL("val", val);
        if (val < 11)
        {
            ERRP("noted", "write 2a");
            errCount++;
        }

        /* clear the fifo and ACK previously written packets */
        ioctlArgs.reg_id = R_ACK;
        ioctlArgs.reg_val = ACK_CLEAR | ACK_DPA;
        ioctl(fd, WRITEREG, (int) &ioctlArgs);

        /* switch to packet mode on */
        ioctl(fd, PACKET, ON);

        /* ackknowledge any previously written packets */
        ioctlArgs.reg_id = R_ACK;
        ioctlArgs.reg_val = ACK_DPA;
        ioctl(fd, WRITEREG, (int) &ioctlArgs);

        PRNT("Testing packet 'O' response...\n");
        sprintf(buf, "%c%c%c%c", SOH, 'O', '0', ETB);
        val = write(fd, buf, 4);
        DVAL("val", val);
        if (val < 4)
        {
            ERRP("noted", "write 2b");
            errCount++;
        }

        ioctl(fd, DEBUG, 0x2);
        PRNT("Requesting Position Fix...\n");
        sprintf(&buf[max], "%c%c%c%c%c%c%c", SOH, 'J', ETB, DLE, 0x37, DLE, ETX);
        val = write(fd, buf, 7);
        DVAL("val", val);
        if (val < 7)
        {
            ERRP("noted", "write 3a");
            errCount++;
        }

        PRNT("Changing GPS parameters...reduced packet traffic...\n");
        sprintf(&buf[max], "%c%c%c%c%c%c%c%c%c%c%c", SOH, 'J', ETB, DLE, 0x35, 0x02, 0x0, 0x04, 0x0, DLE, ETX);
        val = write(fd, buf, 11);
        DVAL("val", val);
        if (val < 11)
        {
            ERRP("noted", "write 2c");
            errCount++;
        }

        /* do this forever... REAL TEST! HAHAHAHAHAHAHAHA */
        do
        {
            if (flag)
            {
                flag = 0;
                ioctl(fd, SIGNAL, OFF);
                DVAL("** bytes read **", max);
                for (val = 0; val < max; val++)
                {
                    if (((buf[val] >= 'A') && (buf[val] <= 'Z')) ||
                        ((buf[val] >= 'a') && (buf[val] <= 'z')) ||
                        ((buf[val] >= '0') && (buf[val] <= '9')))
                        CVAL("buf", buf[val]);
                    else
                        HVAL("buf", (unsigned char) buf[val]);
                }
                max = 0;
                count = 0;
                ioctl(fd, SIGNAL, ON);
            }
            count++;
        } while (count < 50);
    }
    /* turn off interrupts and close driver */
    close(fd);
    logMsg("No more packets in queue.\n", 0, 0, 0, 0, 0, 0);
    logMsg("# errors logged = %d\n", errCount, 0, 0, 0, 0, 0);
    taskDelay(10);
    return OK;
}
