/*
*******************************************************************************
** Copyright (C) 1995 Advanced Processing Laboratories Inc. (AP Labs)
** All rights reserved.
**
** This file and its contents are a product of AP Labs.
** All software distributed by AP Labs is done so under a software license.
** This file may not be copied or modified without execution of the
** appropriate software license agreement with AP Labs or explicit written
** permission of AP Labs.
**
** This file is provided as is, with no warranties of any kind including
** the warranties of design, merchantability and fitness for a particular
** purpose, or arising from a course of dealing, usage or trade practice.
**
** AP Labs shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part thereof.
**
** In no event will AP Labs be liable for any lost revenue or profits
** or other special, indirect and consequential damages, even if AP Labs
** has been advised of the possibility of such damages, as a result of the
** usage of this file and software for which this file is a part.
******************************************************************************
*/

/*
------------------------------------------------------------------------------


File Name:              btfpTest2.c

Project Name:           Datum VxWorks Driver test for BTFP

Company Name:           Advanced Processing Laboratories, Inc.

Author:                 Brandon Warhurst
Creation Date:          September, 12 1995


Functions Contained in this File:


Purpose:                This file contains the code for the BTFP VxWorks
                        GPS test.


Modification History:
        Name     Date   Description
        ----    ------  -------------------------------------
        bpw     091295  File Created
-------------------------------------------------------------------------------
*/

/*
 * Include files
 */
#include "vxWorks.h"
#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "logLib.h"
#include "sysLib.h"
#include "taskLib.h"
#include "bctfp.app.h"

/*
 * Defines
 */
/* some handy print definitions to keep messages consistent */
#define PRNT(x)   logMsg((char *)x,0,0,0,0,0,0);
#define HVAL(x,y) logMsg("Value of %s is 0x%x.\n",(int)x,(unsigned int)y,0,0,0,0)
#define DVAL(x,y) logMsg("Value of %s is %d.\n",(int)x,(unsigned int)y,0,0,0,0)
#define CVAL(x,y) logMsg("Value of %s is %c.\n",(int)x,(unsigned char)y,0,0,0,0)
#define SVAL(x,y) logMsg("%s is %s.\n",(int)x,(int)y,0,0,0,0)
#define ERRP(x,y) logMsg("*** Error: %s at %s ***\n",(int)x,(int)y,0,0,0,0)

/* always need this conversion :) */
#define RAD_TO_DEG  (float)(180.0/3.1415926535898)

/*
 * typedefs
 */

/*
 * Imports
 */

/*
 * Exports
 */

/*
 * Locals
 */
/* global variables (used by main program and interrupt handler) */
int     flag = 0,
        max = 0;
int     fd = 0;
char    buf[1024];

/*
 * Forward declarations
 */
VOID    myhandler(int);
STATUS  go();
float   radToDeg(float);
VOID    floatCvt(char *, float *);
VOID    printMessage(char *, int);

VOID
myhandler(int val)
{

    PRNT("*** called myhandler ***\n");
    max += read(fd, (char *) &buf[max], 512 - max);
    flag = 1;
}

STATUS
go()
{
    STATUS  status;
    int     val,
            errCount = 0,
            maxLockCount = 0;
    float   fval[3];
    args    ioctlArgs;
    static  onceThrough;

    PRNT("----- Bancomm Time Frequency Processor Driver Test -----\n");
    PRNT("Calling btfpDevCreate...\n");
    if (onceThrough != 1)
    {
        status = btfpDevCreate("btfp0", (char *) 0xff00, 5, 25);
        if (status == ERROR)
        {
            ERRP("Couldn't create device", "btfpDevCreate");
            return ERROR;
        }
    }
    onceThrough = 1;

    PRNT("Opening Time Frequency Processor...\n");
    fd = open("btfp0", 0, 0);
    if (fd < 0)
    {
        ERRP("Can't open Time Frequency Processor", "open");
        return ERROR;
    }
    /* taskDelay(sysClkRateGet()*5); */
    ioctl(fd, DEBUG, 0x0);

    PRNT("Going to GPS mode...\n");
    sprintf(buf, "%c%s%c", SOH, "A6", ETB);
    val = write(fd, buf, 4);
    DVAL("val", val);
    if (val < 4)
    {
        ERRP("noted", "write 2");
        errCount++;
    }

    /* change GPS parameters so we can check output */
    PRNT("Changing GPS parameters...reducing packet traffic...\n");
    sprintf(buf, "%c%c%c%c%c%c%c%c%c%c%c", SOH, 'J', ETB, DLE, 0x35, 0x02, 0x0, 0x04, 0x0, DLE, ETX);
    val = write(fd, buf, 11);
    DVAL("val", val);
    if (val < 11)
    {
        ERRP("noted", "write 2a");
        errCount++;
    }
    /* taskDelay(sysClkRateGet()*5); */

    /* clear the fifo and ACK previously written packets */
    ioctlArgs.reg_id = R_ACK;
    ioctlArgs.reg_val = ACK_CLEAR | ACK_DPA;
    ioctl(fd, WRITEREG, (int) &ioctlArgs);

    /* connect a user interrupt handler */
    ioctl(fd, INTCONN, (int) myhandler);

    /* switch to packet mode on */
    ioctl(fd, PACKET, ON);
    /* taskDelay(sysClkRateGet()*5); */

    /* check for freewheeling */
    ioctlArgs.reg_val = 0x0010;
    PRNT("Checking for lock...\n");
    do
    {
        val = R_TIMEREQ;
        ioctl(fd, READREG, (int) &val);
        val = R_TIME0;
        ioctl(fd, READREG, (int) &val);
        taskDelay(sysClkRateGet() * 5);
        maxLockCount++;
    } while ((val & 0x0010) && (maxLockCount < 10));
    /* tell driver we want to be notified of interrupts */
    ioctl(fd, SIGNAL, ON);
    ioctlArgs.reg_id = R_MASK;
    ioctlArgs.reg_val = INT_DPA;
    ioctl(fd, WRITEREG, (int) &ioctlArgs);

    /* ackknowledge any previously written packets */
    ioctlArgs.reg_id = R_ACK;
    ioctlArgs.reg_val = ACK_DPA;
    ioctl(fd, WRITEREG, (int) &ioctlArgs);
    /* taskDelay(sysClkRateGet()*5); */

    if (maxLockCount >= 10)
    {
        PRNT("Can't lock to satellites...aborting...\n");
        PRNT("Getting system message...\n");
        sprintf(buf, "%c%c%c%c%c%c%c", SOH, P_GPS_DATA, ETB, DLE, 0x28, DLE, ETX);
        max = 0;
        flag = 0;
        val = write(fd, buf, 7);
        if (val < 7)
        {
            ERRP("noted", "write 2aa");
            errCount++;
        }
        else
        {
            while (!flag)
                taskDelay(sysClkRateGet());
            for (val = 0; val < max; val++)
            {
                if ((buf[val] == DLE) && (buf[val + 1] == 0x48))
                {
                    printMessage((char *) &buf[val + 2], 22);
                    break;
                }
            }
        }
        close(fd);
        return OK;
    }

    /* ioctl(fd,DEBUG,0x58); */

    PRNT("Got lock...\n");
    max = 0;
    flag = 0;
    PRNT("Requesting Position Fix...\n");
    sprintf(buf, "%c%c%c%c%c%c%c", SOH, 'J', ETB, DLE, 0x37, DLE, ETX);
    val = write(fd, buf, 7);
    DVAL("val", val);
    if (val < 7)
    {
        ERRP("noted", "write 3a");
        errCount++;
    }

    while (!flag)
        taskDelay(sysClkRateGet() * 5);

    ioctl(fd, SIGNAL, OFF);
    /* taskDelay(sysClkRateGet()*5); */
    DVAL("** bytes read **", max);
    for (val = 0; val < max; val++)
    {
        if (buf[val] == DLE && buf[val + 1] == 0x4a)
        {
            PRNT("GPS packet found.\n");
            break;
        }
        HVAL("buf", (unsigned char) buf[val]);
    }
    if (val < max)
    {
        taskDelay(sysClkRateGet() * 10);
        floatCvt(buf + val + 2, (float *) &fval[0]);
        floatCvt(buf + val + 6, (float *) &fval[1]);
        floatCvt(buf + val + 10, (float *) &fval[2]);
        fval[0] = radToDeg(fval[0]);
        fval[1] = radToDeg(fval[1]);
        printf("Lat: %f  Long: %f  Alt: %f\n", fval[0], fval[1], fval[2]);
    }
    /* turn off interrupts and close driver */
    close(fd);
    return OK;
}

/*-------------------------- SUPPORT ROUTINES --------------------------*/

VOID
floatCvt(char *buff, float *buff2)
{
    char   *tmp2;

    tmp2 = (char *) buff2;
    *tmp2 = *(buff + 0);
    *(tmp2 + 1) = *(buff + 1);
    *(tmp2 + 2) = *(buff + 2);
    *(tmp2 + 3) = *(buff + 3);

}

float
radToDeg(float radians)
{
    float   val;
    val = RAD_TO_DEG * radians;
    return val;
}

VOID
printMessage(char *message, int len)
{
    char    msg[1024];
    int     i;

    for (i = 0; i < 1024; i++)
        msg[i] = 0;

    strncpy(msg, message, len);

    logMsg("%s\n", (int) msg, 0, 0, 0, 0, 0);
}
