/*
*******************************************************************************
** Copyright (C) 1995 Advanced Processing Laboratories Inc. (AP Labs)
** All rights reserved.
**
** This file and its contents are a product of AP Labs.
** All software distributed by AP Labs is done so under a software license.
** This file may not be copied or modified without execution of the
** appropriate software license agreement with AP Labs or explicit written
** permission of AP Labs.
**
** This file is provided as is, with no warranties of any kind including
** the warranties of design, merchantability and fitness for a particular
** purpose, or arising from a course of dealing, usage or trade practice.
**
** AP Labs shall have no liability with respect to the infringement of
** copyrights, trade secrets or any patents by this file or any part thereof.
**
** In no event will AP Labs be liable for any lost revenue or profits
** or other special, indirect and consequential damages, even if AP Labs
** has been advised of the possibility of such damages, as a result of the
** usage of this file and software for which this file is a part.
******************************************************************************
*/

/*
------------------------------------------------------------------------------

File Name:      btfpTest3.c

Description:
        This file runs the following tests:
        1) Opens two previously created BTFP devices.
           (Allows for external btfpDevCreate calls)
        2) Displays a menu allowing you to choose
           what function to perform.
        3) Performs the following functions:


Modification History:
        Who     Date            What
        ---     ----            ----
        bpw     9/19/95         created

------------------------------------------------------------------------------
*/

/*
 * Include files
 */
#include "vxWorks.h"
#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "logLib.h"
#include "sysLib.h"
#include "taskLib.h"
#include "bctfp.app.h"          /* Bancomm application header file */

/*
 * Defines
 */

/*
 * typedefs
 */

/*
 * Imports
 */

/*
 * Exports
 */

/*
 * Locals
 */

/*
 * Forward declarations
 */

/*
------------------------------------------------------------------------------
*/

STATUS
sendGPSPacket(int fd, char *packet, int len)
{
    char   *gpsPacket;
    int     stat;

    /* allocate buffer for gps packet */
    gpsPacket = (char *) malloc(sizeof(char) * len + 10);
    if (gpsPacket == NULL)
    {
        logMsg("Out of memory allocating gpsPacket.\n", 0, 0, 0, 0, 0, 0);
        return ERROR;
    }

    /*
     * build the gps packet. User only need send in actual gps packet data
     */
    *gpsPacket = SOH;
    *(gpsPacket + 1) = P_GPS_DATA;
    *(gpsPacket + 2) = ETB;
    *(gpsPacket + 3) = DLE;
    strncpy((char *) (gpsPacket + 4), packet, len);
    *(gpsPacket + 4 + len) = DLE;
    *(gpsPacket + 5 + len) = ETX;

    /* write the gps packet to the board */
    ioctl(fd, PACKET, ON);
    stat = write(fd, gpsPacket, len + 6);
    if (stat < (len + 6))
    {
        logMsg("Error writing GPS packet.\n", 0, 0, 0, 0, 0, 0);
        free(gpsPacket);
        return ERROR;
    }

    ioctl(fd, PACKET, OFF);
    free(gpsPacket);
    return OK;
}

STATUS
sendRegPacket(int fd, char *packet, int len)
{
    char   *outPacket;
    int     stat;

    outPacket = (char *) malloc(strlen(packet) + 2);
    if (outPacket == NULL)
    {
        logMsg("Out of memory allocating outPacket.\n", 0, 0, 0, 0, 0, 0);
        return ERROR;
    }

    sprintf(outPacket, "%c%s%c", SOH, packet, ETB);

    ioctl(fd, PACKET, ON);
    stat = write(fd, outPacket, (strlen(packet) + 2));
    if (stat < (strlen(packet) + 2))
    {
        logMsg("Error writing packet.\n", 0, 0, 0, 0, 0, 0);
        free(outPacket);
        return ERROR;
    }

    free(outPacket);
    return OK;
}

STATUS
clearAck(int fd)
{
    args    ioctlArgs;

    ioctlArgs.reg_id = R_ACK;
    ioctlArgs.reg_val = ACK_INFIFO | ACK_DPA;
    /*
     * clear the acknowledges that might have happened
     */
    ioctl(fd, WRITEREG, (int) &ioctlArgs);
    return OK;
}

STATUS
readTime(int fd)
{
    char    timeStr[40];
    int     stat;
    int     tmp[7];

    ioctl(fd, PACKET, OFF);
    stat = read(fd, timeStr, 40);
    if (stat < 40)
    {
        logMsg("Error reading time.\n", 0, 0, 0, 0, 0, 0);
        return ERROR;
    }

    sscanf(timeStr, "%d   %d   %d   %d   %d   %d   %d",
           &tmp[0],
           &tmp[1],
           &tmp[2],
           &tmp[3],
           &tmp[4],
           &tmp[5],
           &tmp[6]);

    taskDelay(sysClkRateGet() * 2);
    printf("Day %d, %d:%d:%d.%d.%d.%d\n", tmp[0], tmp[1], tmp[2],
           tmp[3], tmp[4], tmp[5], tmp[6]);
    taskDelay(sysClkRateGet() * 2);
    return OK;
}

VOID
ihand1(int arg)
{
    if (arg & INT_1PPS)
        logMsg("handler 1: 1pps\n", 0, 0, 0, 0, 0, 0);
}

VOID
ihand2(int arg)
{
    if (arg & INT_1PPS)
        logMsg("handler 2: 1pps\n", 0, 0, 0, 0, 0, 0);
}

VOID
go(int arg)
{
    args    ioctlArgs;
    int     fd;


    if (arg == 1)
    {
        fd = open("btfp0", 0, 0);
        if (fd <= 0)
        {
            logMsg("Error opening btfp0.\n", 0, 0, 0, 0, 0, 0);
            return;
        }
        ioctl(fd, DEBUG, 0x0);
        ioctl(fd, INTCONN, (int) ihand1);
    }
    else
    {
        fd = open("btfp2", 0, 0);
        if (fd <= 0)
        {
            logMsg("Error opening btfp2.\n", 0, 0, 0, 0, 0, 0);
            return;
        }
        ioctl(fd, DEBUG, 0x0);
        ioctl(fd, INTCONN, (int) ihand2);
    }

    ioctl(fd, SIGNAL, ON);

    sendRegPacket(fd, "A3", 2);
    readTime(fd);

    ioctlArgs.reg_id = R_MASK;
    ioctlArgs.reg_val = INT_1PPS;
    ioctl(fd, WRITEREG, (int) &ioctlArgs);

    taskDelay(sysClkRateGet() * 30);
    close(fd);

}

VOID
start()
{
    btfpDevCreate("btfp0", (void *) 0xff00, 5, 25);
    btfpDevCreate("btfp2", (void *) 0xff80, 6, 27);
    sp(go, 1);
    sp(go, 2);
}
